Package: chron
Version: 2.3-59
Title: Chronological Objects which Can Handle Dates and Times
Description: Provides chronological objects which can handle dates and times.
Authors@R: c(person("David", "James", role = "aut",
                     comment = "S original"),
             person("Kurt", "Hornik", role = c("aut", "trl", "cre"),
                    email = "Kurt.Hornik@R-project.org",
                    comment = c(ORCID = "0000-0003-4198-9911", "R port")),
             person("Gabor", "Grothendieck", role = "ctb",
                    email = "ggrothendieck@gmail.com"),
             person("R Core Team", role = "ctb"))
Depends: R (>= 2.12.0)
Imports: graphics, stats
Suggests: scales, ggplot2
Enhances: zoo
License: GPL-2
NeedsCompilation: yes
Packaged: 2023-01-31 12:41:23 UTC; hornik
Author: David James [aut] (S original),
  Kurt Hornik [aut, trl, cre] (<https://orcid.org/0000-0003-4198-9911>, R
    port),
  Gabor Grothendieck [ctb],
  R Core Team [ctb]
Maintainer: Kurt Hornik <Kurt.Hornik@R-project.org>
Repository: CRAN
Date/Publication: 2023-01-31 13:10:38 UTC
